@@ -17,6 +17,7 @@
 #include "soup-message-private.h"
 #include "soup-message-metrics-private.h"
 #include "soup-uri-utils-private.h"
+#include "content-sniffer/soup-content-sniffer-stream.h"
 
 /**
  * SECTION:soup-message
@@ -72,7 +73,6 @@ typedef struct {
 	guint              msg_flags;
 
 	SoupContentSniffer *sniffer;
-	gsize              bytes_for_sniffing;
 
 	SoupHTTPVersion    http_version, orig_http_version;
 
@@ -93,6 +93,7 @@ typedef struct {
 
 	gboolean is_top_level_navigation;
         gboolean is_options_ping;
+        gboolean force_http1;
         guint    last_connection_id;
         GSocketAddress *remote_address;
 
@@ -1421,12 +1422,14 @@ soup_message_set_connection (SoupMessage    *msg,
 		g_signal_handlers_disconnect_by_data (priv->connection, msg);
                 priv->io_data = NULL;
 		g_object_remove_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
+                soup_connection_set_in_use (priv->connection, FALSE);
 	}
 
 	priv->connection = conn;
 	if (!priv->connection)
 		return;
 
+        soup_connection_set_in_use (priv->connection, TRUE);
         priv->last_connection_id = soup_connection_get_id (priv->connection);
 
 	g_object_add_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
@@ -1665,6 +1668,9 @@ soup_message_is_keepalive (SoupMessage *msg)
 {
 	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
+        if (priv->http_version == SOUP_HTTP_2_0)
+                return FALSE;
+
 	if (priv->status_code == SOUP_STATUS_OK &&
 	    priv->method == SOUP_METHOD_CONNECT)
 		return TRUE;
@@ -2187,7 +2193,7 @@ soup_message_io_finished (SoupMessage *msg)
                 return;
 
         g_assert (priv->connection != NULL);
-        soup_client_message_io_finished (g_steal_pointer (&priv->io_data));
+        soup_client_message_io_finished (g_steal_pointer (&priv->io_data), msg);
 }
 
 void
@@ -2197,7 +2203,7 @@ soup_message_io_pause (SoupMessage *msg)
 
         g_return_if_fail (priv->io_data != NULL);
 
-        soup_client_message_io_pause (priv->io_data);
+        soup_client_message_io_pause (priv->io_data, msg);
 }
 
 void
@@ -2207,7 +2213,7 @@ soup_message_io_unpause (SoupMessage *msg)
 
         g_return_if_fail (priv->io_data != NULL);
 
-        soup_client_message_io_unpause (priv->io_data);
+        soup_client_message_io_unpause (priv->io_data, msg);
 }
 
 gboolean
@@ -2215,7 +2221,7 @@ soup_message_is_io_paused (SoupMessage *msg)
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        return priv->io_data && soup_client_message_io_is_paused (priv->io_data);
+        return priv->io_data && soup_client_message_io_is_paused (priv->io_data, msg);
 }
 
 /**
@@ -2231,7 +2237,7 @@ soup_message_io_in_progress (SoupMessage *msg)
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        return priv->io_data != NULL;
+        return priv->io_data && soup_client_message_io_in_progress (priv->io_data, msg);
 }
 
 void
@@ -2240,7 +2246,7 @@ soup_message_io_run (SoupMessage *msg,
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        soup_client_message_io_run (priv->io_data, blocking);
+        soup_client_message_io_run (priv->io_data, msg, blocking);
 }
 
 gboolean
@@ -2250,7 +2256,7 @@ soup_message_io_run_until_read (SoupMessage  *msg,
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        return soup_client_message_io_run_until_read (priv->io_data, cancellable, error);
+        return soup_client_message_io_run_until_read (priv->io_data, msg, cancellable, error);
 }
 
 void
@@ -2262,7 +2268,7 @@ soup_message_io_run_until_read_async (SoupMessage        *msg,
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        soup_client_message_io_run_until_read_async (priv->io_data, io_priority, cancellable, callback, user_data);
+        soup_client_message_io_run_until_read_async (priv->io_data, msg, io_priority, cancellable, callback, user_data);
 }
 
 gboolean
@@ -2274,14 +2280,14 @@ soup_message_io_run_until_read_finish (SoupMessage  *msg,
 }
 
 gboolean
-soup_message_io_run_until_finish (SoupMessage  *msg,
-                                  gboolean      blocking,
-                                  GCancellable *cancellable,
-                                  GError      **error)
+soup_message_io_skip (SoupMessage  *msg,
+                      gboolean      blocking,
+                      GCancellable *cancellable,
+                      GError      **error)
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        return soup_client_message_io_run_until_finish (priv->io_data, blocking, cancellable, error);
+        return soup_client_message_io_skip (priv->io_data, msg, blocking, cancellable, error);
 }
 
 void
@@ -2303,15 +2309,7 @@ soup_message_io_get_response_istream (SoupMessage  *msg,
 {
         SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
-        return soup_client_message_io_get_response_stream (priv->io_data, error);
-}
-
-SoupContentSniffer *
-soup_message_get_content_sniffer (SoupMessage *msg)
-{
-	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
-
-	return priv->sniffer;
+        return soup_client_message_io_get_response_stream (priv->io_data, msg, error);
 }
 
 void
@@ -2325,12 +2323,29 @@ soup_message_set_content_sniffer (SoupMessage *msg, SoupContentSniffer *sniffer)
 	priv->sniffer = sniffer ? g_object_ref (sniffer) : NULL;
 }
 
-void
-soup_message_set_bytes_for_sniffing (SoupMessage *msg, gsize bytes)
+gboolean
+soup_message_try_sniff_content (SoupMessage  *msg,
+                                GInputStream *stream,
+                                gboolean      blocking,
+                                GCancellable *cancellable,
+                                GError      **error)
 {
-	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        SoupContentSnifferStream *sniffer_stream;
+        const char *content_type;
+        GHashTable *params;
 
-	priv->bytes_for_sniffing = bytes;
+        if (!priv->sniffer)
+                return TRUE;
+
+        sniffer_stream = SOUP_CONTENT_SNIFFER_STREAM (stream);
+        if (!soup_content_sniffer_stream_is_ready (sniffer_stream, blocking, cancellable, error))
+                return FALSE;
+
+        content_type = soup_content_sniffer_stream_sniff (sniffer_stream, &params);
+        soup_message_content_sniffed (msg, content_type, params);
+
+        return TRUE;
 }
 
 GInputStream *
@@ -2622,3 +2637,77 @@ soup_message_set_metrics_timestamp (SoupMessage           *msg,
                 break;
         }
 }
+
+void
+soup_message_set_request_host_from_uri (SoupMessage *msg,
+                                        GUri        *uri)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        char *host;
+
+        if (priv->http_version == SOUP_HTTP_2_0)
+                return;
+
+        host = soup_uri_get_host_for_headers (uri);
+        if (soup_uri_uses_default_port (uri))
+                soup_message_headers_replace (priv->request_headers, "Host", host);
+        else {
+                char *value;
+
+                value = g_strdup_printf ("%s:%d", host, g_uri_get_port (uri));
+                soup_message_headers_replace (priv->request_headers, "Host", value);
+                g_free (value);
+        }
+        g_free (host);
+}
+
+void
+soup_message_update_request_host_if_needed (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        if (priv->http_version == SOUP_HTTP_2_0)
+                return;
+
+        if (soup_message_headers_get_one (priv->request_headers, "Host"))
+                return;
+
+        soup_message_set_request_host_from_uri (msg, priv->uri);
+}
+
+void
+soup_message_force_keep_alive_if_needed (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        if (priv->http_version == SOUP_HTTP_2_0)
+                return;
+
+        /* Force keep alive connections for HTTP 1.0. Performance will
+         * improve when issuing multiple requests to the same host in
+         * a short period of time, as we wouldn't need to establish
+         * new connections. Keep alive is implicit for HTTP 1.1.
+         */
+        if (!soup_message_headers_header_contains (priv->request_headers, "Connection", "Keep-Alive") &&
+            !soup_message_headers_header_contains (priv->request_headers, "Connection", "close") &&
+            !soup_message_headers_header_contains (priv->request_headers, "Connection", "Upgrade")) {
+                soup_message_headers_append (priv->request_headers, "Connection", "Keep-Alive");
+        }
+}
+
+void
+soup_message_set_force_http1 (SoupMessage *msg,
+                              gboolean     force_http1)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        priv->force_http1 = force_http1;
+}
+
+gboolean
+soup_message_get_force_http1 (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return priv->force_http1;
+}
