@@ -112,6 +112,8 @@ test_cleanup (void)
 		apache_cleanup ();
 #endif
 
+        quart_cleanup ();
+
 	if (logger)
 		g_object_unref (logger);
 	if (index_buffer)
@@ -168,14 +170,7 @@ apache_cmd (const char *cmd)
 	int status;
 	gboolean ok;
 
-	server_root = g_test_build_filename (G_TEST_BUILT, "", NULL);
-	if (!g_path_is_absolute (server_root)) {
-		char *abs_server_root;
-
-		abs_server_root = g_canonicalize_filename (server_root, NULL);
-		g_free (server_root);
-		server_root = abs_server_root;
-	}
+	server_root = soup_test_build_filename_abs (G_TEST_BUILT, "", NULL);
 
 	cwd = g_get_current_dir ();
 #ifdef HAVE_APACHE_2_4
@@ -256,6 +251,75 @@ apache_cleanup (void)
 
 #endif /* HAVE_APACHE */
 
+static GSubprocess *quart_proc;
+
+gboolean
+quart_init (void)
+{
+        if (quart_proc)
+                return TRUE;
+
+        GSubprocessLauncher *launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE); // | G_SUBPROCESS_FLAGS_STDERR_SILENCE
+        g_subprocess_launcher_set_cwd (launcher, g_test_get_dir (G_TEST_DIST));
+
+        GError *error = NULL;
+        char *script = soup_test_build_filename_abs (G_TEST_DIST, "http2-server.py", NULL);
+        quart_proc = g_subprocess_launcher_spawn (launcher, &error, script, NULL);
+        g_free (script);
+        g_object_unref (launcher);
+
+        if (error) {
+                g_test_message ("Failed to start quart server: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        GDataInputStream *in_stream = g_data_input_stream_new (g_subprocess_get_stdout_pipe (quart_proc));
+
+        // We don't own the stream, don't break the pipe
+        g_filter_input_stream_set_close_base_stream (G_FILTER_INPUT_STREAM (in_stream), FALSE);
+
+        // Read stdout until the server says it is running
+        while (TRUE) {
+                char *line = g_data_input_stream_read_line_utf8 (in_stream, NULL, NULL, &error);
+
+                if (error) {
+                        g_test_message ("Failed to start quart server: %s", error->message);
+                        g_error_free (error);
+                        g_object_unref (in_stream);
+                        return FALSE;
+                } else if (line == NULL) {
+                        g_test_message ("Failed to start quart server (not installed?)");
+                        g_object_unref (in_stream);
+                        return FALSE;
+                }
+
+                if (g_str_has_prefix (line, " * Running")) {
+                        g_test_message ("Started quart server: %s", line + 3);
+                        g_free (line);
+                        g_object_unref (in_stream);
+                        return TRUE;
+                }
+                g_free (line);
+        }
+}
+
+void
+quart_cleanup (void)
+{
+        if (quart_proc) {
+                GError *error = NULL;
+                g_subprocess_force_exit (quart_proc);
+                g_subprocess_wait (quart_proc, NULL, &error);
+                if (error) {
+                        g_test_message ("Failed to stop quart server: %s", error->message);
+                        g_error_free (error);
+                }
+        }
+
+        g_clear_object (&quart_proc);
+}
+
 SoupSession *
 soup_test_session_new (const char *propname, ...)
 {
@@ -877,6 +941,42 @@ soup_test_get_index (void)
 	return index_buffer;
 }
 
+char *
+soup_test_build_filename_abs (GTestFileType  file_type,
+                              const gchar   *first_path,
+                              ...)
+{
+        const gchar *pathv[16];
+        gsize num_path_segments;
+        char *path;
+        char *path_abs;
+        va_list ap;
+
+        va_start (ap, first_path);
+
+        pathv[0] = g_test_get_dir (file_type);
+        pathv[1] = first_path;
+
+        for (num_path_segments = 2; num_path_segments < G_N_ELEMENTS (pathv); num_path_segments++) {
+                pathv[num_path_segments] = va_arg (ap, const char *);
+                if (pathv[num_path_segments] == NULL)
+                        break;
+        }
+
+        va_end (ap);
+
+        g_assert_cmpint (num_path_segments, <, G_N_ELEMENTS (pathv));
+
+        path = g_build_filenamev ((gchar **) pathv);
+        if (g_path_is_absolute (path))
+                return path;
+
+        path_abs = g_canonicalize_filename (path, NULL);
+        g_free (path);
+
+        return path_abs;
+}
+
 #ifndef G_HAVE_ISO_VARARGS
 void
 soup_test_assert (gboolean expr, const char *fmt, ...)
